# Ports over a map tile from academy into format 6. 
# This likely has to be heavily modified if you're dealing with map tiles that have references to multiple asset bundles.
# After adding an area, make sure it's registered in the caching manifest using sharedassets.py
# Additionally, you can add in music data for that tile and register it in tabledata's areas table

import os
import json
from unitypack.asset import Asset, ObjectPointer, FFOrderedDict, ObjectInfo

APPDATA_PATH = (os.path.expandvars('%userprofile%')) + '/AppData/LocalLow/Unity/Web Player/Cache/FusionFallA/'

mapfile = open(APPDATA_PATH + 'map/BuildPlayer-Map_00_03', 'rb')
sharedfile = open(APPDATA_PATH + 'map/BuildPlayer-Map_00_03.sharedAssets', 'rb')
pathfile = open('info/pathMap.json', 'r')

# The map file itself is a long list of Unity objects (gameobjects, transforms, etc) that reference each other.
# This asset's objects are properly numbered in order as if it were format 6 (even though the file is format 7)
# The shared assets contains just one PreloadData object with a list of external objects to preload.
mapAsset = Asset.from_file(mapfile)
sharedAsset = Asset.from_file(sharedfile)
pathMap = json.loads(pathfile.read())

def replace(obj):
	className = obj.__class__.__name__
	if className == 'GameObject':
		for key, value in obj._obj.items(): obj._obj[key] = replace(value)
		return obj
	elif className == 'ObjectPointer':
		# The map file has references of training grounds bundle, then default resources
		if obj.file_id != 0 and obj.file_id != 2:
			ret = ObjectPointer(None, mapAsset)
			ret.file_id = 1
			ret.path_id = pathMap[str(obj.path_id)]
			print(ret.path_id)
			return ret
		return obj
	# Really long conditional, but I wanted to make sure I knew everything that I was working with
	elif className == 'Transform' or className == 'Animation' or className == 'AnimationClip' or className == 'MeshFilter' or className == 'Mesh' or className == 'Material' or className == 'Texture2D' or className == 'SubMesh' or className == 'Renderer' or className == 'Shader' or className == 'AudioClip' or className == 'EllipsoidParticleEmitter' or className == 'ParticleAnimator' or className == 'ParticleRenderer':
		for key, value in obj._obj.items(): obj._obj[key] = replace(value)
		return obj
	elif className == 'FFOrderedDict':
		for key, value in obj.items(): obj[key] = replace(value)
		return obj
	elif className == 'str' or className == 'int' or className == 'float' or className == 'bool' or className == 'bytes': return obj
	elif className == 'list':
		ret = [0] * len(obj)
		for i in range(len(obj)): ret[i] = replace(obj[i])
		return ret
	elif className == 'tuple':
		ret = [0] * len(obj)
		for i in range(len(obj)): ret[i] = replace(obj[i])
		return tuple(ret)
	elif className == 'NoneType': return obj
	else:
		print(f'Unhandled {className} {obj}')

for i, obj in mapAsset.objects.items():
	obj._contents = replace(obj.contents)
sharedAsset.objects

print(mapAsset.format)
print(mapAsset.asset_refs)
print(sharedAsset.format)
print(sharedAsset.asset_refs)

for ptr in sharedAsset.objects[1].contents['m_Assets']:
	# The sharedassets file has a reference list of unity default resources, then the training grounds bundle
	if ptr.file_id != 1:
		ptr.file_id = 2
		try:
			ptr.path_id = pathMap[str(ptr.path_id)]
		except:
			raise KeyError('Some paths are not mapped!')
			
mapAsset.format = 6
mapAsset.long_object_ids = False

sharedAsset.format = 6
sharedAsset.long_object_ids = False

APPDATA_PATH = (os.path.expandvars('%userprofile%')) + '/AppData/LocalLow/Unity/Web Player/Cache/FusionFall/'

with open(APPDATA_PATH + 'map_5f00_5f03_2eunity3d/BuildPlayer-Map_00_03', 'wb') as f:
	mapAsset.save(f)

with open(APPDATA_PATH + 'map_5f00_5f03_2eunity3d/BuildPlayer-Map_00_03.sharedAssets', 'wb') as f:
	sharedAsset.save(f)