# Backport npcs from academy asset bundles into a version of the game with format 6.
# This was made before convertbundle.py, and it transfers objects in a slightly messier manner (as it is only partially extracting/injecting from bundles, instead of copying over the whole thing)

import os
from unitypack.asset import Asset, ObjectPointer, FFOrderedDict, ObjectInfo

sourcefile = open('./CustomAssetBundle-DongResources_12_15', 'rb')
srcObjects = Asset.from_file(sourcefile).objects
srcContainer = srcObjects[1].contents['m_Container']
srcPreload = srcObjects[1].contents['m_PreloadTable']

APPDATA_PATH = (os.path.expandvars('%userprofile%')) + '/AppData/LocalLow/Unity/Web Player/Cache/FusionFall/'

targetfile = open(APPDATA_PATH + 'world_5fshared_5fpart1_2eresourceFile/CustomAssetBundle-eb037efed74f34db2b2bcdf47626cb76_old', 'rb')
data = Asset.from_file(targetfile)
objects = data.objects
container = objects[1].contents['m_Container']
preloads = objects[1].contents['m_PreloadTable']
# 137 epmetal triangle 140 injustice 142 gatekeeper 143 neverest (colossus) 145 stone scorpion
for targetAssetNum in [153]:#[137,140,142,143,145]:
	targetAssetName = srcContainer[targetAssetNum][0]
	targetAsset = srcContainer[targetAssetNum][1]

	preloadStart = targetAsset['preloadIndex']
	preloadNum = targetAsset['preloadSize']
	assetPointer = targetAsset['asset']
	texturePointer = None

	dataInsertStart = data.next_path_id()
	pathMap = {}

	assetContainerEntry = (targetAssetName, FFOrderedDict())
	assetContainerEntry[1]['preloadIndex'] = len(preloads) # Preload starts at wherever the array ends right now
	assetContainerEntry[1]['preloadSize'] = preloadNum # Load n assets

	# Traverse through game object, convert object pointers along the way
	def replace(obj):
		className = obj.__class__.__name__
		if className == 'GameObject':
			for key, value in obj._obj.items(): obj._obj[key] = replace(value)
			return obj
		elif className == 'ObjectPointer':
			if obj.source_asset != data:
				ret = ObjectPointer(None, data)
				ret.file_id = 0
				ret.path_id = pathMap[obj.path_id]
				return ret
			return obj
		elif className == 'Transform' or className == 'Animation' or className == 'AnimationClip' or className == 'MeshFilter' or className == 'Mesh' or className == 'Material' or className == 'Texture2D' or className == 'SubMesh' or className == 'Renderer' or className == 'Shader':
			for key, value in obj._obj.items(): obj._obj[key] = replace(value)
			return obj
		elif className == 'FFOrderedDict':
			for key, value in obj.items(): obj[key] = replace(value)
			return obj
		elif className == 'bytes': return obj
		elif className == 'str' or className == 'int' or className == 'float' or className == 'bool': return obj
		elif className == 'list':
			ret = [0] * len(obj)
			for i in range(len(obj)): ret[i] = replace(obj[i])
			return ret
		elif className == 'tuple':
			ret = [0] * len(obj)
			for i in range(len(obj)): ret[i] = replace(obj[i])
			return tuple(ret)
		elif className == 'NoneType': return obj
		else:
			print(f'Unhandled {className} {obj}')

	preloadAssets = srcPreload[(preloadStart) : (preloadStart + preloadNum)]
	for preload in preloadAssets:
		if 'texture' in preload.object.asset.name.lower(): 
			texturePointer = preload # Comes from npctexture

		sourceObjectInfo = preload.object
		obj = data.add_object(sourceObjectInfo.type_id)
		obj._contents = sourceObjectInfo.contents

		pathMap[preload.path_id] = obj.path_id # to replace every occurence of the old path id with that of the newly inserted object

		# preload everything?
		newPtr = ObjectPointer(None, data)	
		newPtr.file_id = 0
		newPtr.path_id = obj.path_id
		preloads.append(newPtr)

	assetContainerEntry[1]['asset'] = ObjectPointer(None, data)
	assetContainerEntry[1]['asset'].file_id = 0
	assetContainerEntry[1]['asset'].path_id = pathMap[assetPointer.path_id] # Convert the old asset pointer
	container.append(assetContainerEntry)

	# Handle (possible) texture
	if texturePointer:
		assetContainerEntry = (targetAssetName.replace('kfm', 'dds').replace('mob/', 'texture/'), FFOrderedDict())
		assetContainerEntry[1]['preloadIndex'] = len(preloads)
		assetContainerEntry[1]['preloadSize'] = 1
		assetContainerEntry[1]['asset'] = ObjectPointer(None, data)
		assetContainerEntry[1]['asset'].file_id = 0
		assetContainerEntry[1]['asset'].path_id = pathMap[texturePointer.path_id]
		container.append(assetContainerEntry)

		newPtr = ObjectPointer(None, data)	
		newPtr.file_id = 0
		newPtr.path_id = pathMap[texturePointer.path_id]
		preloads.append(newPtr)

	for i in range(dataInsertStart, dataInsertStart + preloadNum):
		objects[i]._contents = replace(objects[i].contents)

outfile = open(APPDATA_PATH + 'world_5fshared_5fpart1_2eresourceFile/CustomAssetBundle-eb037efed74f34db2b2bcdf47626cb76', 'wb')
data.save(outfile)