# Some functions for editing the sharedassets0.assets found in main.unity3d.
# The most interesting object in here is CachingManifest, where new bundles can be added for loading, and their versions can be edited to make the client reload any bundles if they are updated.

import os
from unitypack.asset import Asset
from unitypack.object import FFOrderedDict

APPDATA_PATH = (os.path.expandvars('%userprofile%')) + '/AppData/LocalLow/Unity/Web Player/Cache/FusionFall/'

CACHE_SECTIONS = ['m_CharacterCreation', 'm_Tutorial', 'm_CharacterSelection', 'm_FreeZone', 'm_FreeZoneComplete', 'm_PaidZone', 'm_PaidZoneComplete']

# I unpacked the original main.unity3d with disunity and renamed the folder so that I can always work off this untouched asset
f = open('D:/Fusion/cdn/main2/sharedassets0.assets', 'rb')
data = Asset.from_file(f)
data.objects

def point(x,y):
	ffdict = FFOrderedDict()
	ffdict['x'] = x
	ffdict['y'] = y
	return ffdict

# Ex: Retro_shared.resourceFile -> retro_5fshared_2eresourceFile
def toFolderPath(filename):
	parts = filename.split('.')
	return parts[0].lower().replace('_', '_5f') + '_2e' + parts[1]

musicdatas = data.objects[1391].contents['musicDatas']
ambientdatas = data.objects[1391].contents['ambientDatas']
cachingmanifest = data.objects[1375].contents

# Adding an extra music or ambient entry WILL NOT WORK if there is not a corresponding entry of the other type to make the lists of equal length.
# Yes, I learned this the hard way
def addMusic(x, y, music='none.ogg', ambient='none.ogg', delay=0, instanced=0):
	entry = FFOrderedDict()
	entry['index'] = len(musicdatas)
	entry['name'] = ''
	entry['points'] = [point(x + 0.0, y + 0.0), point(x + 0.0, y + 512.0), point(x + 512.0, y + 512.0), point(x + 512.0, y + 0.0)]
	entry['fileName'] = music
	entry['delay'] = delay
	entry['bInstanceFlag'] = instanced
	musicdatas.append(entry)
	
	entry = FFOrderedDict()
	entry['index'] = len(ambientdatas)
	entry['name'] = ''
	entry['points'] = [point(x + 0.0, y + 0.0), point(x + 0.0, y + 512.0), point(x + 512.0, y + 512.0), point(x + 512.0, y + 0.0)]
	entry['fileName'] = ambient
	entry['delay'] = delay
	entry['bInstanceFlag'] = instanced
	ambientdatas.append(entry)

def addBundle(key, name, version=776491):
	entry = FFOrderedDict()
	entry['version'] = version
	entry['fileName'] = name
	assert key in cachingmanifest
	cachingmanifest[key].append(entry)

# Since I'm always working off the original sharedassets, I refer to the appdata to find the current version of a bundle and increment it
def update(name):
	bundle = None 

	for key in CACHE_SECTIONS:
		for b in cachingmanifest[key]:
			if b['fileName'] == name:
				bundle = b
				break

	assert bundle != None
	cachedPath = APPDATA_PATH + toFolderPath(name)
	cachedInfo = open(cachedPath + '/__info', 'r')
	bundle['version'] = int(cachedInfo.readline()) + 1
	print(bundle['fileName'], bundle['version'])

addBundle('m_CharacterCreation', 'TrainingGrounds.resourceFile')
del cachingmanifest['m_Tutorial'][-3] # Map_00_02 was in the tutorial zone as a border
addBundle('m_Tutorial', 'Map_00_02.unity3d')
addBundle('m_Tutorial', 'Map_00_03.unity3d')
addBundle('m_Tutorial', 'Map_00_04.unity3d')

del cachingmanifest['m_PaidZoneComplete'][130]
addBundle('m_PaidZone', 'DongResources_15_09.resourceFile')

addMusic(0, 1024, 'intothevoid.ogg', '28 Ambient Green Maw.ogg')
addMusic(0, 1536, 'heavypetting.ogg', '18 Ambient Loch Mess.ogg')
addMusic(0, 2048, 'neversaynever.ogg', '07 Ambient Mojos Volcano.ogg')

for key in CACHE_SECTIONS:
	for b in cachingmanifest[key]:
		cachedPath = APPDATA_PATH + toFolderPath(b['fileName'])
		cachedInfo = open(cachedPath + '/__info', 'r')
		b['version'] = int(cachedInfo.readline())

update('TableData.resourceFile')
update('Retro_shared.resourceFile')
update('RetroMusic.resourceFile')
update('World_shared_part1.resourceFile')
update('TrainingGrounds.resourceFile')
update('Map_00_02.unity3d')
update('Map_00_03.unity3d')
update('Map_00_04.unity3d')

# for i in range(len(cachingmanifest['m_PaidZoneComplete'])):
# 	print(i, cachingmanifest['m_PaidZoneComplete'][i])

# print()

# for k,v in cachingmanifest.items():
# 	print(k,v)

# print()

# for i in range(len(musicdatas)):
# 	print(i, musicdatas[i])

# print()

# for i in range(len(ambientdatas)):
# 	print(i, ambientdatas[i])

outf = open('D:/Fusion/cdn/main/sharedassets0.assets', 'wb')
data.save(outf)